@@ -1,28 +1,26 @@
 from __future__ import annotations
 
+import enum
 from decimal import Decimal
 
 import pytest
-from motor.motor_asyncio import AsyncIOMotorClient as MongoClient
+from pymongo.asynchronous.mongo_client import AsyncMongoClient as MongoClient
 
 import typedmongo.asyncio as mongo
-from typedmongo.marshamallow import MarshamallowObjectId
 
 
-class MongoTable(mongo.Table):
-    __abstract__ = True
-
-    _id: mongo.ObjectIdField = mongo.ObjectIdField(
-        marshamallow=MarshamallowObjectId(required=False)
-    )
-
-
-class Wallet(mongo.Table):
+class Wallet(mongo.Document):
     balance: mongo.DecimalField
 
 
-class User(MongoTable):
+class Gender(enum.Enum):
+    MALE = "m"
+    FEMALE = "f"
+
+
+class User(mongo.MongoDocument):
     name: mongo.StringField
+    gender: mongo.EnumField[Gender]
     age: mongo.IntegerField
     tags: mongo.ListField[str]
     wallet: mongo.EmbeddedField[Wallet]
@@ -42,7 +40,7 @@ async def init_models():
         User,
     )
     yield
-    await User.objects.drop()
+    await User.objects.collection.drop()
 
 
 async def test_use_objects_in_instance():
@@ -57,6 +55,7 @@ async def document_id():
             {
                 "name": "Aber",
                 "age": 18,
+                "gender": "m",
                 "tags": ["a", "b"],
                 "wallet": {"balance": 100},
                 "children": [],
@@ -103,7 +102,7 @@ async def test_find_one_and_command(document_id):
 
     user = await User.objects.find_one_and_replace(
         User._id == document_id,
-        User.load({"name": "Aber", "age": 0}),
+        User.load({"name": "Aber", "age": 0}, partial=True),
         after_document=True,
     )
     assert user is not None
@@ -124,6 +123,7 @@ async def documents_id():
             {
                 "name": "Aber",
                 "age": 18,
+                "gender": "m",
                 "tags": ["a", "b"],
                 "wallet": {"balance": 100},
                 "children": [],
@@ -133,6 +133,7 @@ async def documents_id():
             {
                 "name": "Yue",
                 "age": 18,
+                "gender": "f",
                 "tags": ["y", "u"],
                 "wallet": {"balance": 200},
                 "children": [],
@@ -200,3 +201,29 @@ async def test_filter_expressions(documents_id, expression):
 async def test_projection(documents_id, projection):
     user = await User.objects.find_one(User.age == 18, projection=projection)
     assert user is not None
+
+
+@pytest.mark.parametrize(
+    "shortcut",
+    [
+        mongo.Contains("be"),
+        mongo.Contains("bE", case_sensitive=False),
+        mongo.StartsWith("A"),
+        mongo.StartsWith("a", case_sensitive=False),
+        mongo.EndsWith("r"),
+        mongo.EndsWith("R", case_sensitive=False),
+    ],
+)
+async def test_shortcut(documents_id, shortcut):
+    user = await User.objects.find_one(User.name == shortcut)
+    assert user is not None
+    assert user.name == "Aber"
+
+
+async def test_transaction(documents_id):
+    async with User.objects.use_session():
+        await User.objects.update_many({}, {"$set": {"age": 20}})
+        await User.objects.delete_many(User.name == "Aber")
+        await User.objects.insert_one(
+            User.load({"name": "Aber", "age": 18}, partial=True)
+        )
