@@ -1,3 +1,6 @@
+from typedmongo.expressions import RawExpression
+from typedmongo.shortcuts import Contains, EndsWith, StartsWith
+
 from .client import (
     DeleteMany,
     DeleteOne,
@@ -13,16 +16,26 @@ from .fields import (
     DecimalField,
     DictField,
     EmbeddedField,
-    FieldParamters,
+    EnumField,
     FloatField,
     IntegerField,
     ListField,
+    LiteralField,
     ObjectIdField,
     StringField,
+    UnionField,
 )
-from .table import Index, Table
+from .table import Document, Index, MongoDocument
+
+# Alias for Document, for compatibility with older versions
+Table = Document
+MongoTable = MongoDocument
 
 __all__ = [
+    "RawExpression",
+    "Contains",
+    "StartsWith",
+    "EndsWith",
     "DeleteMany",
     "DeleteOne",
     "InsertOne",
@@ -31,16 +44,19 @@ __all__ = [
     "UpdateOne",
     "initial_collections",
     "Index",
-    "Table",
+    "MongoDocument",
+    "Document",
     "BooleanField",
     "DateTimeField",
     "DecimalField",
     "DictField",
     "EmbeddedField",
-    "FieldParamters",
+    "EnumField",
     "FloatField",
     "IntegerField",
     "ListField",
+    "LiteralField",
     "ObjectIdField",
     "StringField",
+    "UnionField",
 ]
