@@ -14,18 +14,30 @@ def main(just_check: bool = False):
         content = path.read_text()
         content = (
             content.replace(
-                "from motor.motor_asyncio import AsyncIOMotorCollection",
-                "from pymongo.collection import Collection",
+                "from pymongo.asynchronous.client_session import AsyncClientSession",
+                "from pymongo.synchronous.client_session import ClientSession",
             )
             .replace(
-                "from motor.motor_asyncio import AsyncIOMotorDatabase",
-                "from pymongo.database import Database",
+                "from pymongo.asynchronous.collection import AsyncCollection",
+                "from pymongo.synchronous.collection import Collection",
             )
+            .replace(
+                "from pymongo.asynchronous.database import AsyncDatabase",
+                "from pymongo.synchronous.database import Database",
+            )
+            .replace("from pymongo.asynchronous", "from pymongo.synchronous")
             .replace("async def ", "def ")
             .replace("await ", "")
             .replace("async for ", "for ")
             .replace("async with ", "with ")
             .replace("AsyncIterable", "Iterable")
+            .replace(
+                "AsyncGenerator[MongoSession, None]",
+                "Generator[MongoSession, None, None]",
+            )
+            .replace("AsyncGenerator[None, None]", "Generator[None, None, None]")
+            .replace("AsyncGenerator", "Generator")
+            .replace("asynccontextmanager", "contextmanager")
         )
         if just_check:
             if content != new_file_path.read_text():
